#!/usr/bin/env python

import sys, os
if os.path.exists('/usr/share/inkscape/extensions'):
    sys.path.append('/usr/share/inkscape/extensions') # or another path, as necessary
from simplestyle import *
import random, math, uuid
import inkex
try:
    import feedparser
except ImportError:
    inkex.errormsg('This extension requires FeedParser! Please install from: https://pypi.python.org/pypi/feedparser')
    exit()

lineSpacing = 100 # default distance between subsequent lines of text

# attaches the given text element on the given path
def putTextOnPath(text, path):
    pathID = path.attrib['id']
    textPath = inkex.etree.SubElement(text, 'textPath')
    textPath.set(inkex.addNS('href', 'xlink'), '#%s' % (pathID))
    textSpan = inkex.etree.SubElement(textPath, 'tspan')
    textSpan.text = text.text
    text.text = ''

#Searches all sub-elements of the given element for a TSpan
#Returns the first found TSpan or None
def findTSpan(element):
    if element.tag.endswith('tspan'):
        return element
    for e in element.getchildren():
        t = findTSpan(e)
        if t is not None:
            return t
    return None

# returns a merged style string from the styles given in style1 and style2
# In the case of a collision, style1 overrides entries in style2
def mergeStyles(style1, style2):
    s1 = style1.split(';')
    s2 = style2.split(';')
    d1 = {}
    d2 = {}
    
    # construct a dictionary of style elements for each style
    for s in s1:
        tmp = s.split(':')
        d1[tmp[0]] = tmp[1]
    for s in s2:
        tmp = s.split(':')
        d2[tmp[0]] = tmp[1]
        
    # merge the dictionaries
    merged = d2.copy()
    merged.update(d1)

    # write merged dict to string in form: key1:value1;key2:value2;...
    output = ''
    for k in merged.keys():
        output += '%s:%s;' % (k, merged[k])
    return output

# retrieves styles from elements in a given dictionary
# returns a dictionary of <labels, styles>
def getStyles(elements):
    styles = {}
    for name in elements.keys():
        # check to see if this element or one of its subelements is a TSpan so we can use the span's style
        e = findTSpan(elements[name])
        if 'style' in e.attrib:
            styles[name] = mergeStyles(e.attrib['style'], elements[name].attrib['style'])
        else:
            styles[name] = elements[name].attrib['style']

        #styles[name] = elements[name].attrib['style'] if e == None else mergeStyles(e.attrib['style'], elements[name].attrib['style'])

    return styles

# computes relative distances between the given elements
# returns a dictionary of <labels, points>
def getOffsets(elements):
    offsets = {}
    root = None
    for name in elements.keys():
        if root == None: # use first element as root
            root = elements[name]
        
        dx = float(elements[name].get('x')) - float(root.get('x'))
        dy = float(elements[name].get('y')) - float(root.get('y'))
        
        offsets[name] = (dx, dy)

    return offsets

class RssFeedReaderEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("--tab", action="store", type="string", 
            dest="tab", default="object") 
        self.OptionParser.add_option('-u', '--url', action='store', type='string',
            dest='url', default='', help='You must specify a feed URL to pull from!')
        self.OptionParser.add_option('-n', '--nentries', action='store', type='int',
            dest='maxEntries', default=10)
        self.OptionParser.add_option('-t', '--printtitle', action='store', default='true')
        self.OptionParser.add_option('-a', '--printauthor', action='store', default='true')
        self.OptionParser.add_option('-d', '--printdesc', action='store', default='true')
        self.OptionParser.add_option('-l', '--printlink', action='store', default='true')
        self.OptionParser.add_option('-p', '--putonselection', action='store', 
            default='false')
        self.OptionParser.add_option('-S', '--usestyletemplate', action='store', 
            default='false')
        self.OptionParser.add_option('-s', '--separateentries', action='store',
            default='true')

    # appends the list of text elements in text to the given layer, ensuring all text is added below top
    def addTextToLayerAt(self, text, layer, top, selection):
        for t in text:
            t.set('x', str(0))
            t.set('y', str(top))
            layer.append(t)
            if selection is not None:
                putTextOnPath(t, selection)
            top = top + lineSpacing # need to adjust by height of previous element to account for varying font sizes...
        return top

    # appends the list of text elements in text to the given layer, adjusting positions according to offsets
    def addTemplatedTextToLayerAt(self, text, layer, top, offsets):
        ox = 0
        oy = top
        for i in range(len(text)):
            inkex.errormsg(offsets[i])
            text[i].set('x', str(ox + offsets[i][0]))
            text[i].set('y', str(oy + offsets[i][1]))
            layer.append(text[i])
            #top = top + lineSpacing
        return top
    
    # outputs RSS entries as flat text, no formatting. Collects elements selected in extension menu.
    def printEntriesSimple(self, svg, content, maxEntries, separateEntries, putOnSelection):
        printTitle = self.options.printtitle
        printAuthor = self.options.printauthor
        printDesc = self.options.printdesc
        printLink = self.options.printlink
        selection = self.selected.popitem()[1] if (putOnSelection == 'true') and (len(self.selected) == 1) else None
        
        if separateEntries == 'false': # create a layer to hold all entries
            layer = inkex.etree.SubElement(svg, 'g')
            layer.set(inkex.addNS('label', 'inkscape'), 'Feed Entries')
            layer.set(inkex.addNS('groupmode', 'inkscape'), 'layer')

        top = 0
        for i in range(0, maxEntries if len(content.entries) > maxEntries else len(content.entries)):
            output = []
            if separateEntries == 'true': # create a new layer for each entry
                layer = inkex.etree.SubElement(svg, 'g')
                layer.set(inkex.addNS('label', 'inkscape'), '%s' % (content.entries[i].title))
                layer.set(inkex.addNS('groupmode', 'inkscape'), 'layer')
                top = 0

            if printTitle == 'true' and hasattr(content.entries[i], 'title'):
                titleText = inkex.etree.Element(inkex.addNS('text', 'svg'))
                titleText.set(inkex.addNS('label', 'inkscape'), 'Title')
                titleText.text = '%s' % (content.entries[i].title)
                output.append(titleText)
            if printAuthor == 'true' and hasattr(content.entries[i], 'author'):
                authorText = inkex.etree.Element(inkex.addNS('text', 'svg'))
                authorText.set(inkex.addNS('label', 'inkscape'), 'Author')
                authorText.text = '%s' % (content.entries[i].author)
                output.append(authorText)
            if printDesc == 'true' and hasattr(content.entries[i], 'description'):
                descText = inkex.etree.Element(inkex.addNS('text', 'svg'))
                descText.set(inkex.addNS('label', 'inkscape'), 'Description')
                descText.text = '%s' % (content.entries[i].description)
                output.append(descText)
            if printLink == 'true' and hasattr(content.entries[i], 'link'):
                linkText = inkex.etree.Element(inkex.addNS('text', 'svg'))
                linkText.set(inkex.addNS('label', 'inkscape'), 'Link')
                linkText.text = '%s' % (content.entries[i].link)
                output.append(linkText)
        
            top = self.addTextToLayerAt(output, layer, top, selection)

    # outputs RSS entries following a user-defined template. Collects elements based on template content.
    def printEntriesTemplate(self, svg, content, maxEntries, separateEntries):
        template = getStyles(self.selected)
        offsets = getOffsets(self.selected)
        entryHeight = 0

        # compute rough height of each entry (does not account for font size, etc.)
        for k in offsets.keys():
            if offsets[k][1] > entryHeight: entryHeight = offsets[k][1]

        if separateEntries == 'false': # create a layer to hold all entries
            layer = inkex.etree.SubElement(svg, 'g')
            layer.set(inkex.addNS('label', 'inkscape'), 'Feed Entries')
            layer.set(inkex.addNS('groupmode', 'inkscape'), 'layer')

        top = 0
        for i in range(0, maxEntries if len(content.entries) > maxEntries else len(content.entries)):
            output = []
            if separateEntries == 'true': # create a new layer for each entry
                layer = inkex.etree.SubElement(svg, 'g')
                layer.set(inkex.addNS('label', 'inkscape'), '%s' % (content.entries[i].title))
                layer.set(inkex.addNS('groupmode', 'inkscape'), 'layer')

            groupOffsets = []
            for k in template.keys():
                newText = inkex.etree.Element(inkex.addNS('text', 'svg'))
                newText.set(inkex.addNS('label', 'inkscape'), '%s%s' % (k, i)) # labels: title1, title2...
                newText.text = '%s' % (content.entries[i][k])
                newText.attrib['style'] = template[k]
                output.append(newText)
                groupOffsets.append(offsets[k]) #should correspond to ordering in output
        
            self.addTemplatedTextToLayerAt(output, layer, top, groupOffsets)
            top += entryHeight + lineSpacing #increment top by height of this entry plus a little extra    

    def effect(self):
        feedUrl = self.options.url
        maxEntries = self.options.maxEntries
        putOnSelection = self.options.putonselection
        useTemplate = self.options.usestyletemplate
        separateEntries = self.options.separateentries # if true, creates new layer for each entry

        content = feedparser.parse(feedUrl)        
        svg = self.document.getroot()

        if useTemplate == 'true':
            self.printEntriesTemplate(svg, content, maxEntries, separateEntries)
        else:
            self.printEntriesSimple(svg, content, maxEntries, separateEntries, putOnSelection)


effect = RssFeedReaderEffect()
effect.affect()